function [meanimpulse,upperband,lowerband]=ImpulseSummary(MA,ENDO,plotflag,samescale,percent,bandtype,coverprob,shocklabels,varlabels,Convert2Annual,Difference,season)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   PURPOSE:    Computes impulseresponse (IR) functions with corresponding error bands for Bayesian Cointegrated Structural VARs.
%   
%   INPUT:      MA          Cell matrix of draws from the posterior of the MA-coefficient, as computed by impulseresponse2.m.
%               ENDO        Matrix of endogenous variables
%               plotflag    Graph of the IRs is produced if plotflag==1.
%               samescale   if samescale==1, then IRs share common scale across shocks for a given variable.
%               percent     if percent==1, IRs are scaled so that they show the percentage deviation from the initial value of the
%                           variable.
%               bandtype:   0   Normal approximation of the IR posterior (mu +- c*std). Not recommended. IRs usually behave
%                                   like Cauchy variates.
%                           1   Posterior of IR may be arbitrary - IR intervals are equal-tailed.
%                           2   Posterior of IR may be arbitrary - Highest Posterior Density (HPD) regions.
%                           3   Sims-Zha (Econometrica, 1999) method. Not yet implemented.
%               coverprob   Coverage probability of IRs posterior interval.
%               shocklabels Cell array of string labels for the shocks (Optional)
%               varlabels   Cell array of string labels for the variables (Optional)
%
%   OUTPUT:     meanimpulse Cell array of posterior medians of the IRs
%               upperband   Cell array of posterior upper bands of the IRs
%               lowerband   Cell array of posterior lower bands of the IRs
%
%   AUTHOR:     Mattias Villani     Research Department, Sveriges Riksbank
%                                   Department of Statistics, Stockholm University
%                                   e-mail:mattias.villani@stat.su.se  
%
%   FIRST VER:    2004-11-24
%   REVISED:      2007-09-15
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  USER INPUT  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Grayness of uncertainty bands
colorband=[0.65 0.65 0.65;0.459 0.456 1.000;0.071 0.110 0.553]; % Blue/Purple-ish color
factor=1; % Factor that can be used to multiply all impulse responses, sometimes gives a more clear Y-axis.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

coverprob=coverprob(:); % Force to be a column vector
coverprob=flipud(sort(coverprob)); % Arrange in descending order
ncoverprobs=length(coverprob);


p=size(MA,1);
horizon=size(MA,3)-1;
meanimpulse=cell(p,p);
niter=size(MA,4);
%n_std=NGT med erfinv av (1-coverprob)/2
n_std=1; % Temporary. Always one std error bands if bandtype=0.
if nargin>7
    labels=1;
else
    labels=0;
end

if percent==1
    percentfrominitial=ENDO(end,:);
else
    percentfrominitial=100*ones(1,p);
end

if ncoverprobs==0
    upperband=nan;
    lowerband=nan;
end

switch bandtype
case 0 % Normal approximation - symmetric bands,
    for h=0:horizon
        TEMPMEAN=median(MA(:,:,h+1,:),4);
        if ML==0
            TEMPSTD=std(MA(:,:,h+1,:),1,4);
        end
        for i=1:p
            for j=1:p
                meanimpulse{i,j}(h+1)=100*TEMPMEAN(i,j)/percentfrominitial(i);
                if ML==0
                    upperband{i,j}(h+1)=100*(TEMPMEAN(i,j)+n_std*TEMPSTD(i,j))/percentfrominitial(i);
                    lowerband{i,j}(h+1)=100*(TEMPMEAN(i,j)-n_std*TEMPSTD(i,j))/percentfrominitial(i);
                end
            end
        end
    end
    
s=zeros(horizon+1,niter);
case 1 % Arbitrary posterior density of impulse responses - Equal probability mass in both tails
    low=round(((1-coverprob)/2)*niter);
    high=round(niter-((1-coverprob)/2)*niter);
    for j=1:p
        for i=1:p
            for h=0:horizon
                Temp=MA(i,j,h+1,:);
                Impulses(h+1,:)=Temp(:)';
            end
            if Convert2Annual==1 && any(Difference==i)
                Impulses=Diff2AnnualDiff(Impulses,zeros(season-1,niter),season);
            end
            SortedImpulse=sort(Impulses,2);
            meanimpulse{i,j}=factor*100*SortedImpulse(:,round(niter/2))/percentfrominitial(i);
            for v=1:ncoverprobs
                upperband{i,j,v}=factor*100*SortedImpulse(:,high(v))/percentfrominitial(i);
                lowerband{i,j,v}=factor*100*SortedImpulse(:,low(v))/percentfrominitial(i);
            end
        end
    end   
otherwise
end

if plotflag==1
    k=0;
    for i=1:p
        maximum=-inf;
        minimum=inf;
        for j=1:p
            k=k+1;
            subplot(p,p,k)  
            hold on
                for v=1:ncoverprobs
                    y=[upperband{i,j,v}' fliplr(lowerband{i,j,v}')];
                    x=[[0:horizon] fliplr([0:horizon])];
                    a1=patch(x,y,colorband(v,:));
                    
                    set(a1,'faceLighting','phong')
                    set(a1,'facealpha',0.5)
                    set(a1,'edgecolor',min([1.05*colorband(v,:)],[1 1 1]));
                    set(a1,'edgealpha',0.5)
                end
            plot([0:horizon],meanimpulse{i,j},'k');
            if ncoverprobs>0
                maximum=max([maximum upperband{i,j,1}']);
                minimum=min([minimum lowerband{i,j,1}']);
            else
                maximum=max([maximum meanimpulse{i,j}']);
                minimum=min([minimum meanimpulse{i,j}']);
            end
            
            linehandle=line([0 horizon],[0 0]);
            set(linehandle,'linewidth',0.05,'linestyle',':','color',[0.2 0.2 0.2]);
            if i*labels==1
                title(shocklabels{j})
            end
            if labels==1 & j==1
                subplot(p,p,k)
                ylabel(varlabels{i})
            end
            %set(gca,'xticklabel','')
            %set(gca,'xtick',[])
            set(gca,'fontsize',8)
        end
        if samescale==1
            ktemp=k;
            subplot(p,p,ktemp)
            minimum=rounddigit(minimum,3);
            maximum=rounddigit(maximum,3);
            axis([0 horizon minimum maximum]);
            set(gca,'yaxisLocation','right');
            set(gca,'ytick',[minimum maximum]);
            for j=2:p
                subplot(p,p,ktemp-1)
                axis([0 horizon minimum maximum]);
                set(gca,'ytick',[]);
                ktemp=ktemp-1;
            end
        end
    end
end

if samescale~=1
    for i=1:p^2
        subplot(p,p,i)
        axis tight
    end
end

if plotflag==1
    orient landscape
end