function [UpsHat,Pihat,LogLik]=MLEstStructuralVAR(Endo,D,K,Grest,g,IncludeConstant,DfCorrML)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PURPOSE: 	ML estimates for the structural cointegrated VAR model.
%									
% INPUT:	Y
%           X
%           Grest
%           g
%
% OUTPUT:	UpsilonHat      (niter-by-s) matrix of draws from the posterior of the free simultaneous coefficients. s=sum s(i), where 
%                               s(i) are the number of free simultaneous coefficients in the ith equation
%           Pihat
%
% AUTHOR:	Mattias Villani, Department of Statistics, Stockholm University.
%		    E-mail: mattias.villani@stat.su.se.
%
% DATE:		2004-10-21
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[T,p]=size(Endo);

% Adding a constant to D
if IncludeConstant>0 
    D=[D ones(T,1)];
end

% Transform data to multivariate regression form: Y=XB+E.
[Y,X,D]=transform_to_multireg(Endo,D,K);
X=[X D];
[T,p]=size(Y);
q=size(D,2);

if isempty(g) % Default normalization of variables ith coefficient in equation i.
    [G,g]=makeG(Grest); 
else % Use g to normalize
    G=makeG(Grest);
end

% Defines the number of free parameters in each simultaneous equation vector.
si=zeros(p,1);
for i=1:p
    si(i)=size(G{i},2);
end
s=sum(si);
delimit=[0;cumsum(si)];
MX=eye(T)-X*inv(X'*X)*X';

% Chris Sims' Csminwel optimizer
MaxOptAlgoCrit=10^-10;
MaxOptAlgoIter=10000;
LowerOptBound=-inf*ones(s,1);
for j=1:p
    JthEquationsCoeff=1+delimit(j):delimit(j+1);
    LowerOptBound(JthEquationsCoeff(g(j)))=0;
end

% Initial values by projecting recursive estimates on the subspace given by G
% if DfCorrML==1
%     Sigma=Y'*MX*Y/(T-K*p);
% else
%     Sigma=Y'*MX*Y/T;
% end

Pihat=inv(X'*X)*X'*Y;

if DfCorrML==1
    Sigma=(Y-X*Pihat)'*(Y-X*Pihat)/(T-K*p);
else
    Sigma=(Y-X*Pihat)'*(Y-X*Pihat)/T;
end

UpsilonRecursive=inv(chol(Sigma));
S=BlockDiag(G); % Creates big block diagonal restriction matrix from the cell matrix G.
VecUps = ((S'*S)\(S'*vec(UpsilonRecursive)))'; % The projection
UpperOptBound=inf*ones(s,1);
VecUps(VecUps<LowerOptBound')=-VecUps(VecUps<LowerOptBound');

TimeOut=2;
%disp('%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Posterior maximization : Constrained CSMINWEL %%%%%%%%%%%%%%%%%%%%%%%%%%%%')
[fh,VecUpsHat,gh,InvHessian,itct,fcount,exitflag] = csminwelCon('ConcLogLikSVAR',VecUps,10^(-4)*eye(length(VecUps)),[],MaxOptAlgoCrit,MaxOptAlgoIter,LowerOptBound,UpperOptBound,TimeOut,Y,MX,G,delimit,si);
LogLik=-fh;
UpsHat=zeros(p,p);
% Construct Upsilon
for j=1:p
    UpsHat(:,j)=G{j}*VecUpsHat(1+delimit(j):delimit(j+1))';
end
Pihat=inv(X'*X)*X'*Y*UpsHat;